﻿---------------
-- Libraries --
---------------
local TipHooker = LibStub:GetLibrary("LibTipHooker-1.1")
local StatLogic = LibStub:GetLibrary("LibStatLogic-1.2")
local Toolkit = LibStub:GetLibrary("Toolkit-1.0");

--local L = LibStub("AceLocale-3.0"):GetLocale("HealPoints") --Get the localization for our addon
--local ReforgingInfo = LibStub:GetLibrary("LibReforgingInfo-1.0");


--------------------
-- Initialization --
--------------------
HealPointsTooltips = {}
--local tips = HealPointsTooltips
local HP = HealPoints

local lastLink;
local topItemDiff, bottomItemDiff;

function HealPointsTooltips:OnEnable()
	--Called by HealPoints when HealPoints addon is enabled (OnEnable)
	
	-- Hook item tooltips
	--HP:Debug("Hooking HealPoints tooltips");
	TipHooker:Hook(self.ProcessTooltip, "item")
end;

function HealPointsTooltips:OnDisable()
	--Called by HealPoints when HealPoints addon is disabled (OnDisable)
	-- Unhook item tooltips
	--HP:Debug("Unhooking HealPoints tooltips");
	TipHooker:Unhook(self.ProcessTooltip, "item")
end;

--[[
	Callback function called by TipHooker
	@param tooltip The blizzard tooltip frame, supporting all the methods it supports, e.g.
			AddDoubleLine("textLeft", "textRight")
			AddLine("text")
			Show()
		See http://wowprogramming.com/docs/widgets/GameTooltip
	@param name (string) The name of the item we're providing the tooltip for (e.g. "Lavascale Fillet")
	@param link (string,link) The string in-game link of the item we're getting the tooltip for (e.g. "[Lavascale Fillet]")
--]]
function HealPointsTooltips.ProcessTooltip(tooltip, name, link, ...)
	-- Check if we're in standby mode
	if not HP:IsEnabled() then return end
	
	if (HP.db.char.tooltips == 0) then
		return
	end;
	
	assert(tooltip);
	
	if (link ~= lastLink) then
		--HP:Print("Link: "..Toolkit:VarAsString(link)..", lastLink: "..Toolkit:VarAsString(lastLink));
		topItemDiff, bottomItemDiff = HealPointsTooltips:GetHealPointsDifferenceForItem(link)
		lastLink = link;
	end;

	--HealPoints:Print("ProcessTooltip: topItemDiff = "..Toolkit:VarAsString(topItemDiff));
	--if (bottomItemDiff) then
--		HealPoints:Print("ProcessTooltip: bottomItemDiff = "..Toolkit:VarAsString(bottomItemDiff));
--	end;

	if (topItemDiff == nil) then
		return;
	end;
	
	--HP:Print("topItemDiff: "..Toolkit:VarAsString(topItemDiff));
	--HP:Print("bottomItemDiff: "..Toolkit:VarAsString(bottomItemDiff));
	
	local function addLine(text, d1, d2)
		if (d1 == d2) then
			tooltip:AddDoubleLine(text, HealPointsUtil:colorValue(d1),
				NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
				NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		else
			tooltip:AddDoubleLine(text, HealPointsUtil:colorValue(d1).." / "..HealPointsUtil:colorValue(d2),
				NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
				NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		end
	end

	   --[[
		local location = nil
		if itemType == "INVTYPE_FINGER" or itemType == "INVTYPE_TRINKET" then
			location = L[" (Top/Bottom):"]
		elseif itemType == "INVTYPE_WEAPON" then
			location = L[" (Main/Off):"]
		elseif itemType == "INVTYPE_2HWEAPON" then
			location = L[" (Main+Off):"]
		else
			location = ":"
		end
		--]]

	if (bottomItemDiff ~= nil) then --and bottomItemDiff ~= 0) then
		addLine("HealPoints difference, top slot:", topItemDiff, topItemDiff);
		addLine("HealPoints difference, bottom slot:", bottomItemDiff, bottomItemDiff);
	else --if (topItemDiff ~= 0) then -- or topItemDiff ~= 0) then
		addLine("HealPoints difference:", topItemDiff, topItemDiff);
		--tooltip:AddDoubleLine("HealPoints difference: ", Toolkit:VarAsString(topItemDiff));
	end

	--The tooltip is not automatically resized to fit the added line; to do so, call the tooltip's :Show() method after adding lines.	
	tooltip:Show(); 
end;

function HealPointsTooltips:GetHealPointsDifferenceForItem(link)
	-- Get diff tables
	local diffTable1, diffTable2
	--if HealPointsTooltips:GetItemSubType(link) == L["Gems"] then
--		diffTable1 = StatLogic:GetSum(link)
--	else
		diffTable1, diffTable2 = StatLogic:GetDiff(link)
	--end
	
--	HealPoints:Print("GetHealPointsDifferenceForItem - diffTable1 = "..Toolkit:VarAsString(diffTable1));
--	HealPoints:Print("GetHealPointsDifferenceForItem - diffTable2 = "..Toolkit:VarAsString(diffTable2));
	
	-- Check for bags
	if not diffTable1 then
		return nil, nil;
	end

	if (IsEquippedItem(link)) then
		return nil, nil;
	end;
	
	if IsAltKeyDown() then
		--Add the itemLink if they're pressing Shift
		--HP:Print(string.gsub(link, "|", "||")); --escape | to ||
		--HP:Debug("DiffTable:");
		--HP:Print("GetHealPointsDifferenceForItem: "..Toolkit:VarAsString(diffTable1));
	end;

	--Get the player's current stats
	local currentStats = HealPointsCalculator:GetCurrentHealPoints();
	
	local changes;
	local afterStats; --topItemStats, bottomItemStats;
	
	--Calculate new set of stats based on the current stats and the changes
	changes = HealPointsTooltips:BuildChangesTable(diffTable1);
	
	if IsShiftKeyDown() and IsControlKeyDown() then
		HP:Print("Changes: "..Toolkit:VarAsString(changes));
	end;

	local topDiff, bottomDiff; --HealPoints difference in top/bottom slot
	
	afterStats = HealPointsCalculator:ApplyChanges(currentStats, changes);
	HealPointsCalculator:CalculateHealPoints(afterStats);
	topDiff = afterStats.healPoints - currentStats.healPoints;
	
	if (diffTable2) then
		changes = HealPointsTooltips:BuildChangesTable(diffTable2);
		afterStats = HealPointsCalculator:ApplyChanges(currentStats, changes);
		HealPointsCalculator:CalculateHealPoints(afterStats);
		bottomDiff = afterStats.healPoints - currentStats.healPoints;
	end;

	--HP:Print("topDiff = "..Toolkit:VarAsString(topDiff));
	--HP:Print("bottomDiff = "..Toolkit:VarAsString(bottomDiff));
	--HP:Print("currentStats: "..Toolkit:VarAsString(currentStats));
	
	return topDiff, bottomDiff;
end;

--[[
StatLogic already had the ability to build a diffTable that contains the differences between two items.
What we're going to do is copy the supplied diffTable into a new "changes" table, only 
keeping (and also renaming) the attriutes we're interested in.

	Returned table changes attributes that are being altered:
			changes.stamina
			changes.strength
			changes.attackPower
			changes.agility
			changes.intellect
			changes.spirit
			changes.healing
			changes.spellCritRating
			changes.SpellHasteRating
			changes.mana
			changes.manaCost
]]--
function HealPointsTooltips:BuildChangesTable(diffTable)
	--HP:Debug("HealPointsTooltips:BuildChanges()");
	--HP:Print("diffTable="..Toolkit:VarAsString(diffTable));

	assert(diffTable, "HealPointsTooltips:BuildChanges: diffTable is nil");

	--local changes = {};
	local changes = Toolkit:CopyTable(diffTable);

	
	--Attributes
	changes.strength = changes.STR;	--ADD_HEAL_MOD_STR
	changes.STR = nil;
	changes.agility = changes.AGI;	--ADD_HEAL_MOD_AGI
	changes.AGI = nil;
	--changes.stamina = changes.STA; 	--ADD_HEAL_MOD_STA (depricated)
	changes.STA = nil;
	changes.intellect = changes.INT;	--ADD_HEAL_MOD_INT
	changes.INT = nil;
	changes.spirit = changes.SPI;		--ADD_SPELL_CRIT_RATING_MOD_SPI
	changes.SPI = nil;					--ADD_HEAL_MOD_SPI

	--Spell
	changes.spellPower = changes.HEAL;
	changes.HEAL = nil;
	changes.spellCritRating = changes.SPELL_CRIT_RATING;
	changes.SPELL_CRIT_RATING = nil;
	changes.spellHasteRating = changes.SPELL_CRIT_RATING;
	changes.SPELL_CRIT_RATING = nil;
	changes.mana = changes.MOD_MANA;
	changes.MOD_MANA = nil;
	changes.manaCostModifier = changes.MOD_MANA_COST;
	changes.MOD_MANA_COST = nil;
	changes.spellHasteRating = changes.SPELL_HASTE_RATING;
	changes.SPELL_HASTE_RATING = nil;

	changes.attackPower = changes.AP;	--ADD_HEAL_MOD_AP
	changes.AP = nil;

	return changes
end
